/**
 * 
 */
package de.charite.compbio.asdpex.cmd;

import de.charite.compbio.asdpex.Hg38altLociSeletorOptions;
import de.charite.compbio.asdpex.exceptions.AltLociSelectorException;
import de.charite.compbio.asdpex.exceptions.CommandLineParsingException;
import de.charite.compbio.asdpex.exceptions.HelpRequestedException;

/**
 * Super class for all commands, i.e. the classes implementing one Command execution
 *
 * @author Marten Jäger <marten.jaeger@charite.de>
 *
 */
public abstract class AltLociSelectorCommand {

	/** Configuration to use for the command execution. */
	protected Hg38altLociSeletorOptions options;

	/**
	 * Initialize the JannovarCommand.
	 *
	 * @param argv
	 *            command line arguments to use
	 * @throws CommandLineParsingException
	 *             on problems with the command line
	 * @throws HelpRequestedException
	 *             if the user requested help through the command line parameters
	 */
	public AltLociSelectorCommand(String[] args) throws CommandLineParsingException, HelpRequestedException {
		this.options = parseCommandLine(args);
	}

	/**
	 * Function for parsing the command line.
	 *
	 * @param argv
	 *            command line to parse, as in the program's main function
	 * @return {@link Hg38altLociSeletorOptions} with the programs' configuration
	 * @throws CommandLineParsingException
	 *             on problems with the command line
	 * @throws HelpRequestedException
	 *             when the user requested the help page
	 */
	protected abstract Hg38altLociSeletorOptions parseCommandLine(String[] args)
			throws CommandLineParsingException, HelpRequestedException;

	public abstract void run() throws AltLociSelectorException;

}
